
# q-expansion of newform 1344.4.a.y, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1344
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [3], [10], [7], [12], [-30], [34], [-148], [152], [106], [304], [114], [202], [-116], [224], [274], [660], [-382], [-12], [-552], [-614], [880], [108], [-86], [1426], [810], [1032], [-900], [-454], [18], [-1072], [1084], [2426], [-684], [-3598], [2680], [-1422], [-220], [1992], [-4286], [4516], [2218], [3136], [-3198], [3714], [-3256], [4612], [2976], [1084], [2746], [-5062], [320], [-6974], [356], [4770], [2712], [-6942], [-4720], [1490], [-7926], [8596], [-438], [-5140], [4296], [2714], [4314], [2284], [9650], [-4068], [3922], [-414], [5160], [-11056], [7762], [-12228], [336], [-1966], [-4606], [-13038], [7290], [5836], [-2398], [-17568], [4354], [-14616], [-14516], [-14462], [3610], [3138], [6896], [1356], [9136], [4072], [-12420], [13892], [-20408], [3666], [-5046], [18628], [5594], [-13372], [-15478], [-12964], [10506], [6268], [-5790], [-8332], [-23310], [18184], [-9398], [-20192], [-11758], [-2310], [-19356], [-7768], [-11102], [-22612], [-1656], [-16070], [-21740], [31642], [-27518], [7290], [-3684], [-24788], [-15574], [24338], [17760], [3704], [32354], [-8860], [8168], [-25600], [-14878], [9690], [-10670], [-2326], [-10612], [-41166], [-34182], [10308], [37362], [-30808], [43596], [8578], [-41304], [11498], [21210], [-3580], [-10144], [-45926], [-9038], [30692], [22104], [-42212], [-24512], [19912], [6802], [55482], [-3630], [32372], [-43286], [-15864], [9956], [28818], [40920], [3072], [11306], [-942], [-5014], [31244], [-42222], [984], [-12518], [34656], [64010], [21196], [-22254], [-2312], [31346], [27568], [5548], [26546], [4378], [-21312], [26346], [-28262], [-17924], [24282], [-36624], [22370], [13116], [17092], [77394], [8180], [-53574], [-71918], [60682], [16546], [25432], [43402], [49440], [31450], [-85214], [-53148], [3426], [18032], [17028], [-58150], [-85804], [76434], [6618], [-55304], [20244], [47288], [-8118], [3984], [-13150], [86472], [-13942], [-71310], [-49112], [-39934], [-89472], [106796], [-45854], [37098], [-67120], [22360], [-77076], [55130], [-61796], [93760], [35866], [-3868], [-24048], [14978], [16242], [-103060], [49464], [-88364], [36148], [5624], [24842], [107922], [14584], [-21728], [83932], [73068], [34256], [-68678], [28802], [-90632], [-70374], [89522], [-25980], [-85790], [-69116], [49386], [-25638], [-44272], [-60], [-86166], [67106], [-98574], [-34196], [-91990], [77002], [-49428], [-49038], [-28030], [15188], [-21142], [42864], [21714], [-76312], [25756], [95834], [-20518], [82668], [40416], [61496], [-36840], [-95286], [-7196], [60288], [-654], [120530], [68664], [-43038], [-18390], [145676], [-16918], [-62620], [-93782], [-94222], [79488], [-85046], [-140892], [-68276], [-50422], [74778], [-133776], [44820], [-107940], [-60478], [-44244], [104210], [76888], [-30782], [-94288], [-1310], [-38270], [-5884], [-83560], [79930], [-147364], [-106208], [-86270], [132978], [-70724], [-62614], [67842], [-32480], [-164422], [166466], [-68764], [184436], [-91344], [-158286], [59658], [-2646], [137680], [56780], [92476], [110740], [68714], [71842], [-15910], [24832], [171762], [-176118], [82650], [56792], [-17462], [207108], [93338], [144236], [158032], [-175334], [-142772], [-80294], [124746], [-83232], [77202], [72522], [-210576], [52420], [187794], [-221448], [73378], [5754], [-43488], [192012], [50940], [-208390], [-118398], [149448], [237514], [75892], [1780], [12816], [-171694], [-164984], [-35718], [147884], [46608], [-25630], [99170], [55834], [32018], [-203686], [-35960], [92322], [-3380], [119352], [28880], [36826], [7468], [93232], [51458], [-61558], [150492], [-40452], [155992], [36842], [41008], [58666], [230300], [127650], [192834], [102530], [87216], [208602], [97498], [175880], [-81206], [-215534], [-86052], [192300], [129522], [-148254], [-265028], [159476], [-2310], [241698], [174496], [10120], [-307326], [186280], [33066], [-8726], [-158832], [-83964], [211530], [184914], [158900], [19146], [121444], [126712]]
